--[[
***************************************
Raw-Belichtungsreihe inkl. Entwicklung
in der Kamera und Einrechnung von
Tonwertkurven
***************************************
(c) msl
Licence: GPL
thx fudgey, mattkime
***************************************
@title Beli-Reihe - RAW-Entw.
@param a Schrittweite      [x*1/3Ev]
@default a 3
@param b Anzahl Bilder    [n]
@default b 3
@param c Entwicklung? [0=nein][1=ja]
@default c 1
@param d Bildmischung [0=sum][1=ds]
@default d 1
@param e RAWs lschen [0=nein][1=ja]
@default e 1
@param f WAV lschen  [0=nein][1=ja]
@default f 1
@param g Tonwertkurve [0=nein][1-4]
@default g 0
@param h nur Entwickl.[0=nein][1=ja]
@default h 0
@param i Shoot-Pause  [0=nein][1=ja]
@default i 0
--]]

-- functions

function log(...)
	io.write(...)
	io.write("\n")
end

function fastshoot()
  	press("shoot_half")
  	repeat
    		sleep(1)
  	until get_shooting() == true
  	press("shoot_full")
  	release("shoot_full")
  	release("shoot_half")
  	repeat
   	 	sleep(1)
  	until get_shooting() ~= true
end

function wait()
	repeat
		print("weiter mit [SET]")
		wait_click()
	until is_pressed "set"
	print("Start in 2 Sekunden...")
	sleep (2000)
end

function ev_display(nn)
	ev_print = {"-4","-3 2/3","-3 1/3","-3","-2 2/3","-2 1/3","-2",
	"-1 2/3","-1 1/3","-1","-2/3","-1/3","0","1/3","2/3","1","1 1/3",
	"1 2/3","2","2 1/3","2 2/3","3","3 1/3","3 2/3","4"}
	ev_output = ev_print[nn + 13].." Ev"
return ev_output
end

function getImageDirs() 
        local dcimList = os.listdir("A/DCIM", false) 
        local i=1 
        if(dcimList) then 
            table.sort(dcimList) 
            while i<=#dcimList do 
                    if string.upper(string.sub(dcimList[i],4))~="CANON" then 
                        table.remove(dcimList,i) 
                    else i=i+1 end 
            end 
        end 
return dcimList 
end

function getImageList()
	local imgTable = {}
	local dcimList = getImageDirs()
	local dirCount = table.getn(dcimList)
	local i = 0
	while ( dirCount ~= i) do
		i = i + 1
		local imgDirList = os.listdir("A/DCIM/"..dcimList[i], false)
		local imgCount = table.getn(imgDirList)
		table.sort(imgDirList)
		local a = 0
		while (imgCount ~= a) do
			a = a + 1
			table.insert(imgTable,"A/DCIM/"..dcimList[i].."/"..imgDirList[a])
		end
	end
return imgTable
end

function filter(searchString)
	local imageList = getImageList()
	local count = table.getn(imageList)
	local i = 0
	while i ~= count do
		i = i + 1
		if(string.find(imageList[i],searchString,17, true) == nil) then
			table.remove(imageList,i)
			i = i - 1
			count = count - 1
		end
	end
return imageList
end

function restore()
	set_raw(raw_state)
	set_ev(ev_state)
	if c_state ~= nil then set_curve_state(c_state) end
	log("\nscript canceled!!!")
	logfile:close()
end

--start script

if a<0 then a=0 end
if a>6 then a=6 end
if b<0 then b=3 end
if a<2 then if b>7 then b=7 end else if b>5 then b=5 end end
if c<0 or c>1 then c=1 end
if d<0 or d>1 then d=1 end
if e<0 or e>1 then e=0 end
if f<0 or f>1 then f=0 end
if g<0 or g>4 then g=0 end
if h<0 or h>1 then h=0 end
if i<0 or i>1 then i=0 end

step      =a
num  	  =b
dev       =c
operation =d
del_raw   =e
del_wav   =f
curve     =g
only_dev  =h
pause     =i

raw_state = get_raw()
ev_state  = get_ev()/32
c_state   = get_curve_state()

dev_string      = {"develop: off","develop: on"}
op_string       = {"sum","average"}
del_raw_string  = {"RAW delete: off","RAW delete: on"}
del_wav_string  = {"WAV delete: off","WAV delete: on"}
raw_string      = {"RAW: off","RAW: on"}
curve_string    = {"NONE","CUSTOM","+1EV","+2EV","AutoDR"}
only_dev_string = {"complete","only develop"}

log_path = "A/CHDK/LOGS/"
log_file_name = "log.log"
log_name = log_path..log_file_name
logfile=io.open(log_name,"wb")
io.output(logfile)

log(os.date("%x - %X"), " raw merge\n")
log("cam status: ", raw_string[raw_state + 1], " | ", ev_display(ev_state), "\n")
log("number of pics: ", num, "  step wide *1/3Ev: ", step)
log(dev_string[dev + 1], " ", op_string[operation + 1])
log(del_raw_string[del_raw + 1])
log(del_wav_string[del_wav + 1])
log("curve: ", curve_string[curve + 1])
log(only_dev_string[only_dev + 1], "\n")

if c_state == nil and curve > 0 then
	print("Kamera untersttzt keine")
	print("Tonwertkurven!")
	log("No curve function available!")
	sleep(2000)
end

print("Start in 2 Sekunden...")
sleep(2000)

-- bracketing shooting
if only_dev == 0 then
	set_raw(1)
	n=1
	for i=1, num do
		ev = (i/2 * n) * step
		print("Bild", i, "von", num, ev_display(ev))
		log("pic ", i, " of ", num, "  ", ev_display(ev))
		set_ev(ev*32)
		if pause == 1 then wait() end
		fastshoot()
		if n==1 then n=-1 else n=1 end
	end
	set_ev(ev_state)
	set_raw(raw_state)
end

-- search for last raw files
log("\nsearch for the last ", num, " files\n")
search    = "CRW"
list_crw  = filter(search)
count_crw = table.getn(list_crw)
if count_crw < num then
	log("Found only ", count_crw, " files.\n")
	num = count_crw
end


result_crw = {}
a=1
for i=1, num do
	result_crw[i] = list_crw[count_crw - i+1]
end
table.sort(result_crw)

count_result = table.getn(result_crw)
for i=1, count_result do
	log(result_crw[i])
end

-- merge
if result_crw[1] ~= nil then
    log("\nmerge: ",op_string[operation + 1], "\n")
	raw_merge_start(operation)
	for i=1, num do
		print("lade:", string.sub(result_crw[i], 17))
		log("load: ", result_crw[i])
		raw_merge_add_file(result_crw[i])
	end
	print("erzeuge WAV-Datei")
	log("\ncreate WAV file")
	raw_merge_end()
else
	print("Keine RAW-Dateien!")
	log("No RAW files for merging found!")
end

-- search last wav file
log("\nsearch for the last wav file\n")
search    = "WAV"
list_wav  = filter(search)
count_wav = table.getn(list_wav)
if list_wav[1] ~= nil then
	log(list_wav[count_wav])
else
	log("No WAV file found!")
end

-- develop
if dev == 1 then
	if list_wav[1] ~= nil then
	    set_ev(0)
		if c_state ~= nil then set_curve_state(curve) end
		log("\nmake JPG\n")
		set_raw_develop(list_wav[count_wav])
		fastshoot()
		search    = "JPG"
		list_jpg  = filter(search)
		listcount = table.getn(list_jpg)
		log(list_jpg[listcount])
		set_ev(ev_state)
		if c_state ~= nil then set_curve_state(c_state) end
	else
		print("Keine WAV-Datei!")
		log("No file for development!")
	end
end

-- search last jpg number
search    = "JPG"
list_jpg  = filter(search)
count_jpg = table.getn(list_jpg)
last_jpg  = string.sub(list_jpg[count_jpg], 21, string.len(list_jpg[count_jpg])-4)
log("\nlast pic number: ", last_jpg, "\n")

-- delete raw files
if del_raw == 1 then
    print("RAWs lschen.")
    log("delete RAW files\n")
	if result_crw[1] ~= nil then
		for i=1, num do
		log("delete: ", result_crw[i])
	    os.remove(result_crw[i])
	    end
	else
		print("keine RAW-Dateien!")
		log("No RAW files found!\n")
	end
end

-- delete wav file or rename to *.cr2
if list_wav[1] ~= nil then
	if del_wav == 1 then
		print("WAV lschen.")
		log("delete: ", list_wav[count_wav])
		os.remove(list_wav[count_wav])
	else
		print("WAV in CR2 umbenennen.")
		new_name = string.sub(list_wav[count_wav], 1, string.len(list_wav[count_wav])-8)..last_jpg..".CR2"
		log("\nrename: ", list_wav[count_wav], "\nto:     ", new_name)
		os.rename(list_wav[count_wav], new_name)
	end
else
	print("keine WAV-Datei!")
	log("No WAV file for deleting or renaming!")
end
		
-- ready
print("-------fertig--------")
log("\nready -> ", os.date("%X"))
logfile:close()
play_sound(0)

-- rename logfile
log_num = 1
while os.stat(log_path..last_jpg.."_"..log_num..".log") do
	log_num = log_num + 1
end
os.rename(log_name, log_path..last_jpg.."_"..log_num..".log")
